/**
 * Copyright (C) 2009-2010 Gauvain Pocentek <gpocentek@linutop.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "gtk-extensions.h"

/* Toggle button linked to a boolean item */
static void
on_btn_toggled_yes (GtkToggleButton *btn, ConfigItem *item)
{
    gboolean b = gtk_toggle_button_get_active (btn);
    wk_config_item_set_value (item, GINT_TO_POINTER ((int) b), FALSE);
}

static void
on_btn_toggled_no (GtkToggleButton *btn, ConfigItem *item)
{
    gboolean b = !gtk_toggle_button_get_active (btn);
    wk_config_item_set_value (item, GINT_TO_POINTER ((int) b), FALSE);
}

void
wk_gtk_toggle_button_link_boolean (GtkToggleButton *btn, ConfigItem *item)
{
    g_return_if_fail (GTK_IS_TOGGLE_BUTTON (btn));
    g_return_if_fail (item != NULL);
    g_return_if_fail (
        wk_config_item_get_type (item) == CONFIG_ITEM_TYPE_BOOLEAN);

    gtk_toggle_button_set_active (btn,
        (gboolean) GPOINTER_TO_INT (wk_config_item_get_value (item)));
    g_signal_connect (G_OBJECT (btn), "toggled",
                      G_CALLBACK (on_btn_toggled_yes), item);
}

void
wk_gtk_toggle_button_link_not_boolean (GtkToggleButton *btn, ConfigItem *item)
{
    g_return_if_fail (GTK_IS_TOGGLE_BUTTON (btn));
    g_return_if_fail (item != NULL);
    g_return_if_fail (
        wk_config_item_get_type (item) == CONFIG_ITEM_TYPE_BOOLEAN);

    gtk_toggle_button_set_active (btn,
        (gboolean) GPOINTER_TO_INT (wk_config_item_get_value (item)));
    g_signal_connect (G_OBJECT (btn), "toggled",
                      G_CALLBACK (on_btn_toggled_no), item);
}

GtkWidget *
wk_gtk_check_button_new (const gchar *label, ConfigItem *item, gboolean r)
{
    GtkWidget *btn;

    if (label)
        btn = gtk_check_button_new_with_label (label);
    else
        btn = gtk_check_button_new ();

    if (r)
        wk_gtk_toggle_button_link_not_boolean (GTK_TOGGLE_BUTTON (btn), item);
    else
        wk_gtk_toggle_button_link_boolean (GTK_TOGGLE_BUTTON (btn), item);

    return btn;
}

/* Entry linked to string value */
static void
on_entry_changed (GtkEntry *entry, ConfigItem *item)
{
    wk_config_item_set_value (
        item, (gpointer) g_strdup (gtk_entry_get_text (entry)), TRUE);
}

void
wk_gtk_entry_link_string (GtkEntry *entry, ConfigItem *item)
{
    g_return_if_fail (GTK_IS_ENTRY (entry));
    g_return_if_fail (item != NULL);
    g_return_if_fail (
        (wk_config_item_get_type (item) == CONFIG_ITEM_TYPE_STRING)
        || (wk_config_item_get_type (item) == CONFIG_ITEM_TYPE_STRING_FROM_URI));

    gtk_entry_set_text (entry, (gchar *) wk_config_item_get_value (item));
    g_signal_connect (G_OBJECT (entry), "changed",
                      G_CALLBACK (on_entry_changed), item);
}

GtkWidget *
wk_gtk_entry_new (ConfigItem *item)
{
    GtkWidget *entry;

    entry = gtk_entry_new ();
    wk_gtk_entry_link_string (GTK_ENTRY (entry), item);

    return entry;
}

/* Spin button linked to int value */
static void
on_spin_btn_changed (GtkSpinButton *btn, ConfigItem *item)
{
    wk_config_item_set_value (item,
        GINT_TO_POINTER ((int) gtk_spin_button_get_value (btn)),
        FALSE);
}

void
wk_gtk_spin_button_link_int (GtkSpinButton *btn, ConfigItem *item)
{
    g_return_if_fail (GTK_IS_SPIN_BUTTON (btn));
    g_return_if_fail (item != NULL);
    g_return_if_fail (wk_config_item_get_type (item) == CONFIG_ITEM_TYPE_INT);

    gtk_spin_button_set_value (btn,
        GPOINTER_TO_INT (wk_config_item_get_value (item)));
    g_signal_connect (G_OBJECT (btn), "value-changed",
        G_CALLBACK (on_spin_btn_changed), item);
}

GtkWidget *
wk_gtk_spin_button_new_with_range (gdouble min, gdouble max, gdouble step,
                                   ConfigItem *item)
{
    GtkWidget *btn;

    btn = gtk_spin_button_new_with_range (min, max, step);
    wk_gtk_spin_button_link_int (GTK_SPIN_BUTTON (btn), item);

    return btn;
}

