/**
 * Copyright (C) 2009-2010 Gauvain Pocentek <gpocentek@linutop.com>
 *
 * This code is very much inspired from Thunar
 * Copyright (c) 2006 Benedikt Meurer <benny@xfce.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <glib.h>
#include <glib/gprintf.h>
#include <dbus/dbus-glib.h>
#include <dbus/dbus-glib-bindings.h>

#include "wk-dbus-service.h"
#include "wk-dbus-service-info.h"

#include "main.h"
#include "wk-browser.h"

static void wk_dbus_service_finalize (GObject *object);

G_DEFINE_TYPE(WkDBusService, wk_dbus_service, G_TYPE_OBJECT);

static void
wk_dbus_service_class_init (WkDBusServiceClass *klass)
{
    GObjectClass *gobject_class;
    gobject_class = G_OBJECT_CLASS (klass);
    gobject_class->finalize = wk_dbus_service_finalize;

    dbus_g_object_type_install_info (WK_TYPE_DBUS_SERVICE,
                                     &dbus_glib_wk_dbus_service_object_info);
}

static void
wk_dbus_service_init (WkDBusService *dbus_service)
{
    GError *error = NULL;

    dbus_service->connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
    if (G_LIKELY (dbus_service->connection != NULL))
    {
        dbus_g_connection_register_g_object (dbus_service->connection,
                                             "/com/linutop/WebKiosk",
                                             G_OBJECT (dbus_service));
        dbus_bus_request_name (dbus_g_connection_get_connection (
                                    dbus_service->connection),
                               "com.linutop.WebKiosk",
                               DBUS_NAME_FLAG_REPLACE_EXISTING,
                               NULL);
    }
    else
    {
        g_fprintf (stderr,
                   _("Failed to connect to the D-BUS session bus: %s\n"),
                   error->message);
        g_error_free (error);
    }
}

static void
wk_dbus_service_finalize (GObject *object)
{
    WkDBusService *dbus_service = WK_DBUS_SERVICE (object);

    if (G_LIKELY (dbus_service->connection != NULL))
        dbus_g_connection_unref (dbus_service->connection);

    (*G_OBJECT_CLASS (wk_dbus_service_parent_class)->finalize) (object);
}

gboolean
wk_dbus_service_add_time (WkDBusService *dbus_service,
                          guint seconds,
                          GError **error)
{
    WkBrowser *browser;

    browser = WK_BROWSER (wk_browser_get (NULL));
    wk_browser_add_time (WK_BROWSER (browser), seconds);

    return TRUE;
}

gboolean
wk_dbus_service_start_session (WkDBusService *dbus_service, GError **error)
{
    WkBrowser *browser;

    browser = WK_BROWSER (wk_browser_get (NULL));
    wk_browser_start_session (WK_BROWSER (browser));

    return TRUE;
}

gboolean
wk_dbus_service_stop_session (WkDBusService *dbus_service, GError **error)
{
    WkBrowser *browser;

    browser = WK_BROWSER (wk_browser_get (NULL));
    wk_browser_clear_session (WK_BROWSER (browser), TRUE);

    return TRUE;
}

gboolean
wk_dbus_service_quit (WkDBusService *dbus_service, GError **error)
{
    WkBrowser *browser;

    browser = WK_BROWSER (wk_browser_get (NULL));
    wk_browser_cleanup (browser);

    gtk_main_quit ();

    return TRUE;
}

gboolean
wk_dbus_service_show_item (WkDBusService *dbus_service,
                           guint          id,
                           GError       **error)
{
    WkBrowser *browser;

    browser = WK_BROWSER (wk_browser_get (NULL));
    wk_browser_show_item (browser, id);

    return TRUE;
}

gboolean
wk_dbus_service_show_item_next (WkDBusService *dbus_service, GError **error)
{
    WkBrowser *browser;

    browser = WK_BROWSER (wk_browser_get (NULL));
    wk_browser_show_next (browser);

    return TRUE;
}

gboolean
wk_dbus_service_show_item_prev (WkDBusService *dbus_service, GError **error)
{
    WkBrowser *browser;

    browser = WK_BROWSER (wk_browser_get (NULL));
    wk_browser_show_prev (browser);

    return TRUE;
}

gboolean
wk_dbus_service_get_playlist (WkDBusService *dbus_service,
                              gchar **playlist,
                              GError **error)
{
    WkBrowser *browser;

    browser = WK_BROWSER (wk_browser_get (NULL));
    wk_browser_get_playlist (browser, playlist);

    return TRUE;
}

