/**
 * Copyright (C) 2009-2010 Gauvain Pocentek <gpocentek@linutop.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef WK_DBUS_SERVICE_H
#define WK_DBUS_SERVICE_H

#include <glib-object.h>

#include <dbus/dbus-glib-lowlevel.h>
#include <dbus/dbus-glib.h>
#include <dbus/dbus.h>

typedef struct _WkDBusService WkDBusService;
typedef struct _WkDBusServiceClass WkDBusServiceClass;

GType wk_dbus_service_get_type (void);

struct _WkDBusService
{
    GObject __parent__;
    DBusGConnection *connection;
};

struct _WkDBusServiceClass
{
    GObjectClass __parent__;
};

#define WK_TYPE_DBUS_SERVICE         (wk_dbus_service_get_type ())
#define WK_DBUS_SERVICE(obj)         (G_TYPE_CHECK_INSTANCE_CAST ((obj), WK_TYPE_DBUS_SERVICE, WkDBusService))
#define WK_DBUS_SERVICE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), WK_TYPE_TIMER, WkDBusServiceClass))

gboolean wk_dbus_service_quit (WkDBusService *dbus_service,
                               GError       **error);

gboolean wk_dbus_service_add_time      (WkDBusService *dbus_service,
                                        guint          seconds,
                                        GError       **error);
gboolean wk_dbus_service_start_session (WkDBusService *dbus_service,
                                        GError       **error);
gboolean wk_dbus_service_stop_session  (WkDBusService *dbus_service,
                                        GError       **error);

gboolean wk_dbus_service_show_item      (WkDBusService *dbus_service,
                                         guint          id,
                                         GError       **error);
gboolean wk_dbus_service_show_item_next (WkDBusService *dbus_service,
                                         GError       **error);
gboolean wk_dbus_service_show_item_prev (WkDBusService *dbus_service,
                                         GError       **error);

gboolean wk_dbus_service_get_playlist (WkDBusService *dbus_service,
                                       gchar        **playlist,
                                       GError       **error);

#endif /* !WK_DBUS_SERVICE_H */
